<?php
/**
 * Plugin Name: Gallery Only
 * Plugin URI: http://spiresoftware.com.au/blog/gallery-only/
 * Description: This plugin replaces the default gallery with clean looking carousel that utilises the <a href="http://www.gougouzian.fr/projects/jquery/moodular/">Moodular</a> jQuery plugin.
 * Version: 1.1
 * Author: Hammy Goonan
 * Author URI: http://hammygoonan.com
 *
 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU 
 * General Public License version 2, as published by the Free Software Foundation.  You may NOT assume 
 * that you can use any other version of the GPL.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 * even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * @package GalleryOnly
 * @version 1.0
 * @author Hammy Goonan <hammy@spiresoftware.com.au>
 * @copyright Copyright (c) 2008 - 2011, Hammy Goonan
 * @link http://spiresoftware.com.au
 * @license http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 */

	## Initiate the Admin side of things
	add_action('admin_menu', 'my_plugin_menu');
	add_option('go_height', '400', '', 'yes');
	add_option('go_width', '600', '', 'yes');
	add_option('go_direction', 'left', '', 'yes');
	add_option('go_auto', 1, '', 'yes');
	add_option('go_controls', 'button', '', 'yes');
	add_option('go_effects', 'slide', '', 'yes');
	add_option('go_continuous', 1, '', 'yes');
	add_option('go_speed', 2000, '', 'yes');
	add_option('go_dispTimeout', 1000, '', 'yes');
	add_option('go_float', 'right', '', 'yes');
	
	function my_plugin_menu(){
		add_options_page('Gallery Only Administration', 'Gallery Only', 'manage_options', 'gallery_only', 'menu_display');
	}
	function menu_display(){
		# check user is allowed in
		if (!current_user_can('manage_options'))  {
			wp_die( __('You do not have sufficient permissions to access this page.') );
		}
		print("<h1>Gallery Only Administration</h1>");
		print("<p>You can change the gallery options by editing the values below.</p>");
		
		# Save dimentions
		if(isset($_POST['go_height']) && is_numeric($_POST['go_height'])){
			update_option('go_height', $_POST['go_height']);
		}
		elseif(isset($_POST['go_height']) && !is_numeric($_POST['go_height'])){
			print('<div style="color: red">The height must be a number</div>');
		}
		if(isset($_POST['go_width']) && is_numeric($_POST['go_width'])){
			update_option('go_width', $_POST['go_width']);
		}
		elseif(isset($_POST['go_height']) && !is_numeric($_POST['go_width'])){
			print('<div style="color: red">The width must be a number</div>');
		}
		
		# save options
		if(!empty($_POST)){
			update_option('go_float', $_POST['go_float']);
			if(isset($_POST['go_direction'])) update_option('go_direction', $_POST['go_direction']);
			update_option('go_auto', (isset($_POST['go_auto'])) ? 1 : 0);
			if(isset($_POST['go_controls'])) update_option('go_controls', $_POST['go_controls']);
			if(isset($_POST['go_effects'])) update_option('go_effects', $_POST['go_effects']);
			update_option('go_continuous', (isset($_POST['go_continuous']) == 'on') ? 1 : 0);
			if( !empty($_POST['go_speed']) && is_numeric($_POST['go_speed']) ){
				update_option('go_speed', $_POST['go_speed']);
			}
			elseif( !empty($_POST['go_speed']) && !is_numeric($_POST['go_speed']) ){
				print('<div style="color: red">The \'transition speed\' must be a number</div>');
			}
			if( !empty($_POST['go_dispTimeout']) && is_numeric($_POST['go_dispTimeout']) ){
				update_option('go_dispTimeout', $_POST['go_dispTimeout']);
			}
			elseif( !empty($_POST['go_dispTimeout']) && !is_numeric($_POST['go_dispTimeout']) ){
				print('<div style="color: red">The \'time between animations\' must be a number</div>');
			}
		}
		# get variables
		$dimensions = determine_dimensions();

		print('<h2>Gallery Dimentions</h2>');
		print('<table>');
		print('<form name="go_form" method="post" action="">');
		print('<tr>
			<td><label for="go_height">Height (px):</label></td>
			<td><input type="text" name="go_height" value="' . $dimensions['height'] . '" /></td>
		</tr>');
		print('<tr>
			<td><label for="go_width">Width (px):</label></td>
			<td><input type="text" name="go_width" value="' . $dimensions['width'] . '" /></td>
		</tr>');
		print('<tr>
			<td><label for="go_float">Image Alignment:</label></td>
			<td><select name="go_float">');
			$select_options =  array('left', 'right');
			foreach($select_options as $option){
				print ($dimensions['float'] == $option) ? '<option value="' . $option . '" selected="selected">' . $option . '</option>' :
				'<option value="' . $option . '">' . $option . '</option>';
			}
		print('</select></td>
		</tr>');
		print('</table>');
		
		print('<h2>Gallery Options</h2>');
		print('<table>');
		print('<tr>
			<td><label for="go_direction">Direction:</label></td>
			<td><select name="go_direction">');
			$select_options =  array('top', 'bottom', 'left', 'right');
			foreach($select_options as $option){
				print ($dimensions['direction'] == $option) ? '<option value="' . $option . '" selected="selected">' . $option . '</option>' :
				'<option value="' . $option . '">' . $option . '</option>';
			}
		print('</select></td>
		</tr>');
		print('<tr>
			<td><label for="go_auto">Automatically scroll</label></td>
			<td>');
		print ($dimensions['auto'] == 1) ? '<input type="checkbox" name="go_auto" value="' . $dimensions['auto'] . '" checked />' : '<input type="checkbox" name="go_auto" />';
		print('</td>
		</tr>');
		print('<tr>
			<td><label for="go_controls">Controls:</label></td>
			<td><select name="go_controls">');
			$control_options =  array('button', 'keys', 'index', 'click', 'wheel', 'stopOver');
			foreach($control_options as $option){
				print ($dimensions['controls'] == $option) ? '<option value="' . $option . '" selected="selected">' . $option . '</option>' :
				'<option value="' . $option . '">' . $option . '</option>';
			}
		print('</select></td>
		</tr>');
		print('<tr>
			<td><label for="go_effects">Transition Effects:</label></td>
			<td><select name="go_effects">');
			$control_options =  array('slide', 'fade', 'reflection');
			foreach($control_options as $option){
				print ($dimensions['effects'] == $option) ? '<option value="' . $option . '" selected="selected">' . $option . '</option>' :
				'<option value="' . $option . '">' . $option . '</option>';
			}
		print('</select></td>
		</tr>');
		print('<tr>
			<td><label for="go_continuous">Continuous</label></td>
			<td>');
		print ($dimensions['continuous'] == 1) ? '<input type="checkbox" name="go_continuous" value="' . $dimensions['continuous'] . '" checked />' : '<input type="checkbox" name="go_continuous" value="' . $dimensions['continuous'] . '" />';
		print('</td>
		</tr>');
		print('<tr>
			<td><label for="go_speed">Transition Speed (ms)</label></td>
			<td><input type="text" name="go_speed" value="' . $dimensions['speed'] .'" /></td>
		</tr>');
		print('<tr>
			<td><label for="go_dispTimeout">Time between animations (ms)</label></td>
			<td><input type="text" name="go_dispTimeout" value="' . $dimensions['dispTimeout'] .'" /></td>
		</tr>');
		print('</table>');
		?><input type="submit" name="submit" class="button-primary" value="<?php esc_attr_e('Save Changes'); ?>" /><?php
	}
	function determine_dimensions(){
		$height = get_option('go_height');
		$width = get_option('go_width');
		$direction = get_option('go_direction');
		$auto = get_option('go_auto');
		$controls = get_option('go_controls');
		$effects = get_option('go_effects');
		$continuous = get_option('go_continuous');
		$speed = get_option('go_speed');
		$dispTimeout = get_option('go_dispTimeout');
		$float = get_option('go_float');
		return $values = array(
			'width' => $width,
			'height' => $height,
			'direction' => $direction,
			'auto' => $auto,
			'controls' => $controls,
			'effects' => $effects,
			'continuous' => $continuous,
			'speed' => $speed,
			'dispTimeout' => $dispTimeout,
			'float' => $float);
	}
	

 
 
###############################
###							###
###			Front End		###
###							###
###############################

	load_js(); // loads the relevant Javascript files 	
	remove_shortcode('gallery');
	add_shortcode('gallery', 'gallery_only');
	
	function load_js(){
		# load relevant Javascript files;
		wp_enqueue_script('jquery');
		wp_enqueue_script('moodular', plugin_dir_url( __FILE__ ) . 'moodular.js');
		wp_enqueue_script('moodular.controls', plugin_dir_url( __FILE__ ) . 'moodular.controls.js');
		wp_enqueue_script('moodular.effects', plugin_dir_url( __FILE__ ) . 'moodular.effects.js');
		wp_enqueue_script('javascript', plugin_dir_url( __FILE__ ) . 'go_javascript.js');
		wp_enqueue_style('style', plugin_dir_url( __FILE__ ) . 'go_style.css');
	} 	
	function gallery_only(){
 	  	global $post, $wp_locale;
		$image_urls = get_image_url($post->ID);
		// see what happens if you just return a block of code
		return display($image_urls);
	}

	function get_image_url($id){
		$query_images_args = array(
			'post_type' => 'attachment',
			'post_mime_type' =>'image',
			'post_status' => 'inherit',
			'posts_per_page' => -1,
			'post_parent' => $id,
			'order' => 'ASC',
			'orderby' => 'menu_order'
		);
		$query_images = new WP_Query( $query_images_args );
		$images = array();
		foreach ( $query_images->posts as $image) {
			$images[]= $image->guid;
		}
		return $images;
	}
	function display($urls){
		# take an array of URLs and display them. Requires an array of urls and an array of height/width
		$dimensions = determine_dimensions();
		$gallery = '<div id="go_gallery" class="go_gallery"><ul style="width: ' . $dimensions['width'] . 'px; height: ' . $dimensions['height'] . 'px;">';
		$gallery .= "<li style=\"width: " . $dimensions['width'] . "px; height: " . $dimensions['height'] . "px;\"><img src=\"" . plugin_dir_url( __FILE__ ) . "loading.gif\" style=\"float: " . $dimensions['float'] . "\" class=\"go_loading_image\" /></li>";
		foreach($urls as $url){
			$gallery .= "<li style=\"width: " . $dimensions['width'] . "px; height: " . $dimensions['height'] . "px;\"><img src=\"" . $url . "\" style=\"float: " . $dimensions['float'] . "\" class=\"go_gallery_images\" /></li>";
		}
		$gallery .= ('</ul>');
		if($dimensions['controls'] == 'button'){
			$gallery .= '<div class="go_nav" style="width: ' . $dimensions['width'] . 'px">';
			$gallery .= '<a href="#" class="go_prev">Prev</a>';
			$gallery .= '<a href="#" class="go_next">Next</a>';
			$gallery .= '</div>';
		}
		$gallery .= '</div>';
		$gallery .= '<div class="go_clear"></div>';
		return $gallery;
	}
	# Ajax call
	
	add_action('wp_ajax_go_variables', 'go_variables');
	add_action('wp_ajax_nopriv_go_variables', 'go_variables');
	wp_localize_script( 'javascript', 'Ajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' ) ) );
	function go_variables(){
		$options = determine_dimensions();
		$options['auto'] = (bool)$options['auto'];
		$options['continuous'] = (bool)$options['continuous'];
		$options['speed'] = (int)$options['speed'];
		$options['dispTimeout'] = (int)$options['dispTimeout'];
		unset($options['width']);
		unset($options['height']);
		$options['api'] = true;
		header( "Content-Type: application/json" );
		echo json_encode($options);
		exit;
	}
?>